"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var lodash_throttle_1 = __importDefault(require("lodash.throttle"));
var Utils_1 = require("../util/Utils");
require("./SkillBar.scss");
var SkillBar = /** @class */ (function (_super) {
    __extends(SkillBar, _super);
    function SkillBar(props) {
        var _this = _super.call(this, props) || this;
        var clientSide = typeof window !== 'undefined';
        _this.listener = lodash_throttle_1.default(_this.handleScroll.bind(_this), 200);
        if (clientSide && window.addEventListener) {
            window.addEventListener('scroll', _this.listener);
        }
        _this.state = {
            collapsed: true,
            lastVisibility: { partially: false, completely: false },
            clientSide: clientSide
        };
        return _this;
    }
    SkillBar.posTop = function () {
        if (typeof window.pageYOffset !== 'undefined') {
            return window.pageYOffset;
        }
        else if (document && document.documentElement && document.documentElement.scrollTop) {
            return document.documentElement.scrollTop;
        }
        else if (document && document.body && document.body.scrollTop) {
            return document.body.scrollTop;
        }
        return 0;
    };
    SkillBar.prototype.componentDidMount = function () {
        if (this.state.clientSide && this.node) {
            this.setState({
                elementBottom: this.node.getBoundingClientRect().bottom + SkillBar.posTop(),
                elementTop: this.node.getBoundingClientRect().top + SkillBar.posTop()
            }, this.handleScroll);
            this.handleScroll();
        }
    };
    SkillBar.prototype.componentWillUnmount = function () {
        this.removeListener();
    };
    SkillBar.prototype.removeListener = function () {
        if (this.state.clientSide) {
            window.removeEventListener('scroll', this.listener);
        }
    };
    SkillBar.prototype.visibilityHasChanged = function (visible) {
        return visible.completely !== this.state.lastVisibility.completely ||
            visible.partially !== this.state.lastVisibility.partially;
    };
    SkillBar.prototype.isMovingIntoView = function (visible) {
        return !this.state.lastVisibility.completely && visible.completely;
    };
    SkillBar.prototype.handleScroll = function () {
        var _this = this;
        var visible = this.isVisible();
        if (this.state.collapsed && this.visibilityHasChanged(visible) && this.isMovingIntoView(visible)) {
            setTimeout(function () {
                _this.setState({ collapsed: false });
            }, this.props.animationDelay);
            this.removeListener();
            this.setState({ lastVisibility: visible });
        }
        else {
            this.setState({ lastVisibility: visible });
        }
    };
    SkillBar.prototype.isVisible = function () {
        var viewBottom = window.pageYOffset + window.innerHeight;
        var viewTop = window.pageYOffset;
        var offset = this.props.offset;
        var elementBottom = this.state.elementBottom || 0;
        var elementTop = this.state.elementTop || 0;
        var middleOfView = window.pageYOffset + (window.innerHeight / 2);
        if (elementBottom - elementTop > window.innerHeight - (2 * offset)) {
            var completely = (elementTop < middleOfView + offset && elementBottom > middleOfView - offset);
            var partially = completely || (((elementTop > middleOfView + offset && elementTop < viewBottom) ||
                (elementBottom < middleOfView - offset && elementBottom > viewTop)));
            return {
                completely: completely,
                partially: partially
            };
        }
        return {
            completely: (elementBottom < viewBottom - offset && elementBottom > viewTop + offset) &&
                (elementTop > viewTop + offset && elementTop < viewBottom - offset),
            partially: (elementBottom < viewBottom && elementBottom > viewTop) || (elementTop > viewTop && elementTop < viewBottom)
        };
    };
    SkillBar.prototype.getSkillBarColor = function (skill, index) {
        var colors = this.props.colors;
        if (skill.color && skill.color.bar && Utils_1.Utils.isString(skill.color.bar)) {
            return skill.color.bar;
        }
        else if (colors && colors.bar) {
            if (Utils_1.Utils.isString(colors.bar)) {
                return colors.bar;
            }
            return Utils_1.Utils.getHSL(colors.bar, index, this.props.skills.length);
        }
        return '';
    };
    SkillBar.prototype.getTitleColor = function (skill, index, entry) {
        var colors = this.props.colors;
        if (skill.color && skill.color.title && skill.color.title[entry] && Utils_1.Utils.isString(skill.color.title[entry])) {
            return skill.color.title[entry];
        }
        else if (colors && colors.title && colors.title[entry]) {
            if (Utils_1.Utils.isString(colors.title[entry])) {
                return colors.title[entry];
            }
            return Utils_1.Utils.getHSL(colors.title[entry], index, this.props.skills.length);
        }
        return '';
    };
    SkillBar.prototype.render = function () {
        var _this = this;
        var collapsed = this.state.collapsed;
        var skills = this.props.skills;
        return (react_1.default.createElement("div", { ref: function (node) { return _this.node = node; } }, skills.map(function (skill, index) {
            return (react_1.default.createElement("div", { key: skill.type, className: "skillbar", style: {
                    height: "" + Utils_1.Utils.getHeight(_this.props.height),
                    lineHeight: "" + Utils_1.Utils.getHeight(_this.props.height)
                } },
                react_1.default.createElement("div", { className: "skillbar-title", style: {
                        color: "" + _this.getTitleColor(skill, index, 'text'),
                        background: "" + _this.getTitleColor(skill, index, 'background')
                    } },
                    react_1.default.createElement("span", null, skill.type)),
                react_1.default.createElement("div", { className: "skillbar-bar " + (collapsed ? 'collapsed' : ''), style: {
                        background: "" + _this.getSkillBarColor(skill, index),
                        width: "calc((100% - 110px) * (" + skill.level + " * 0.01))",
                        transition: "width " + _this.props.animationDuration + "ms ease-in-out"
                    } }),
                react_1.default.createElement("div", { className: "skillbar-percent" },
                    skill.level,
                    "%")));
        })));
    };
    SkillBar.defaultProps = {
        offset: 25,
        height: 35,
        animationDuration: 3000,
        animationDelay: 1000
    };
    return SkillBar;
}(react_1.default.Component));
exports.default = SkillBar;
